#include <graph/server/gui/chart/chart.h>
#include <graph/server/gui/mainwindow.h>
#include <qabstractseries.h>
#include <qlineseries.h>
#include <QIcon>
#include <iostream>

namespace Graph::GUI {
ChartWidget::ChartWidget(QWidget* parent, const chart_spec_t& spec) : QListWidgetItem(), spec(spec) {
  view = new QChartView(parent);
  chart = new QChart();

  // chart
  if (spec.legend)
    chart->legend()->show();
  else
    chart->legend()->hide();

  chart->setTitle(spec.title);

  // axis
  x = new QValueAxis();
  y = new QValueAxis();

  if (!spec.axis.x.auto_limit)
    x->setRange(spec.axis.x.min, spec.axis.x.max);
  if (!spec.axis.y.auto_limit)
    y->setRange(spec.axis.y.min, spec.axis.y.max);

  x->setTitleText(spec.axis.x.label);
  y->setTitleText(spec.axis.y.label);

  chart->addAxis(x, Qt::AlignBottom);
  chart->addAxis(y, Qt::AlignLeft);

  view->setRenderHint(QPainter::Antialiasing);
  view->setChart(chart);
  view->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
  setSizeHint(view->sizeHint() * 4);

  // context menu
  view->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
  view->connect(view, &QChartView::customContextMenuRequested, view, [this](const QPoint& pos) {
    ContextMenu menu(nullptr, this);
    menu.exec(view->mapToGlobal(pos));
  });
}

ChartWidget::~ChartWidget() {
  // delete view;
  // delete chart;
  // delete x;
  // delete y;
}

void ChartWidget::clear() {
  for (QAbstractSeries* series : chart->series()) {
    ((QLineSeries*)series)->removePoints(0, ((QLineSeries*)series)->count());
  }
}

ContextMenu::ContextMenu(QWidget* parent, ChartWidget* chartWidget) : QMenu(parent) {
  remove = new QAction(tr("&Remove"), this);
  connect(remove, &QAction::triggered, this, [chartWidget]() {
    MainWindow::getInstance()->removeChart(chartWidget);
  });
  addAction(remove);

  clear = new QAction(tr("&Clear"), this);
  connect(clear, &QAction::triggered, this, [chartWidget]() { chartWidget->clear(); });
  addAction(clear);

  edit = new QAction(tr("&Edit"), this);
  connect(edit, &QAction::triggered, this, []() {
    // TODO: implement chart editing
  });
  addAction(edit);
}

ContextMenu::~ContextMenu() {}
} // namespace Graph::GUI